      PROGRAM demo
*
*BOD
*INPUT
**ozone_column
*     real 
*     The ozone profile is scaled to ozone_column given in DU
*     (Dobson units). If ozone_column is negative, the profile is
*     used as is.
*FILES
*     param.inc                Include file
*EOD
*
      IMPLICIT LOGICAL (A-Z)    ! To nearly get strong typing
      INCLUDE '../src_f/param.inc'
*
      INTEGER ndens, nlambda_t, nlev, nlyr
      REAL aer_dtau(maxcly,maxwvn), aer_gg(maxcly,maxwvn),
     $     aer_ssa(maxcly,maxwvn), albedo, day_of_year,
     $     dens(0:maxcly,maxden), 
     $     ice_dtau(maxcly,maxwvn), ice_gg(maxcly,maxwvn), 
     $     ice_ssa(maxcly,maxwvn), lambda_t(maxwvn), 
     $     press(0:maxcly), temper(0:maxcly),
     $     sza, wc_dtau(maxcly,maxwvn), wc_gg(maxcly,maxwvn),
     $     wc_ssa(maxcly,maxwvn), zd(0:maxcly)
      REAL dfdt(maxulv, maxwvn), flup(maxulv, maxwvn),
     $     rfldir(maxulv, maxwvn), rfldn(maxulv, maxwvn), 
     $     uavg(maxulv, maxwvn)
*
* High resolution variables
*
      REAL fbeam( maxwvn_h),
     $     flup_h(maxulv, maxwvn_h), lambda_h(maxwvn_h), 
     $     rfldir_h(maxulv, maxwvn_h), rfldn_h(maxulv, maxwvn_h), 
     $     uavg_h(maxulv, maxwvn_h)
*
      REAL eccentricity
*
*  Variables
*
      CHARACTER*100 filenm_atm, filenm_solar
      INTEGER error, ifna, ifnb, iv, lu, lu_start, nlambda_h,
     $     nlambda_t, nx
      REAL array(maxarray_x, maxarray_y), ecc_fact, ozone_column,
     $     wvn_b, wvn_l, wvn_step_h,
     $     wvn_step_t
*
* INPUT
*
      albedo      = 0.0
      day_of_year = 164
      ozone_column= 300.0
      sza         = 30.0
*
      wvn_b       = 280.0
      wvn_l       = 320.0
      wvn_step_t  = 0.5
*
      filenm_solar = '../data/solar_flux/atlas3'
      filenm_atm   = '../data/atmmod/afglus.dat'
*
* END INPUT
*
*
* Correction factor for Earth-Sun distance
*
      ecc_fact = 1.0
      IF ( day_of_year .GT. 0. ) THEN
         ecc_fact = eccentricity(day_of_year) 
      ENDIF
*
* Read extraterrestrial spectrum
*
      nx = 2
      CALL no_blanks_indices(filenm_solar, ifna, ifnb)
      CALL read_array(array, error, maxarray_x, nx, ny,
     $     filenm_solar(ifna:ifnb))
      IF ( error .NE. 0 ) THEN
         WRITE(*,*)'read_array reports error:', error
         STOP
      ENDIF
      nlambda_h = 1
      iv = 1
      DO WHILE ( array(1,iv) .LE. wvn_l  ) 
         IF ( array(1,iv) .GE. wvn_b) THEN
            lambda_h(nlambda_h) = array(1,iv)
            fbeam(nlambda_h)    = array(2,iv)*ecc_fact
            nlambda_h           = nlambda_h + 1
         ENDIF
         iv = iv + 1
      ENDDO
      nlambda_h           = nlambda_h - 1
*
* Read in model atmosphere 
*
      ndens = 6
      CALL no_blanks_indices(filenm_atm, ifna, ifnb)
      CALL read_atm(dens, filenm_atm(ifna:ifnb), ndens, nlev, press,
     $     temper, zd)
      nlyr = nlev - 1
*
* Scale ozone column
*
      IF ( ozone_column .GE. 0.0 ) THEN
         CALL scale_ozone(dens(0,2), nlyr, ozone_column, zd) 
      ENDIF
*
* Calculate radiation field with fbeam = 1.0, not exactly the same as 
* the transmittance, but no extraterrestrial spectrum involved.
*
* Set wavelength grid for the transmittance calculation
      nlambda_t   = int((wvn_l-wvn_b)/wvn_step_t)
      DO iv = 1, nlambda_t
         lambda_t(iv) = wvn_b + float(iv)*wvn_step_t
      ENDDO
      CALL trans(aer_dtau, aer_gg, aer_ssa, albedo, dens, 
     $     dfdt, flup, ice_dtau, ice_gg, ice_ssa, lambda_t,
     $     ndens, nlambda_t, nlyr, press, rfldir, rfldn, sza,
     $     temper, uavg, wc_dtau, wc_gg, wc_ssa, zd)
*
      lu_start = 15
      WRITE(*,901) (zd(lu-1),lu=lu_start,nlyr+1)
      DO iv = 1, nlambda_t
         WRITE(*,902) lambda_t(iv), (uavg(lu,iv),lu=lu_start,nlyr+1)
      ENDDO
*
* Interpolate to arbitrary wavelength grid using qubic splines.
*
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nlev, flup, flup_h)
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nlev, rfldir, rfldir_h)
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nlev, rfldn, rfldn_h)
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nlev, uavg, uavg_h)
*
*
      lu_start = 15
      WRITE(*,901) (zd(lu-1),lu=lu_start,nlyr+1)
      DO iv = 1, nlambda_h
         WRITE(*,902) lambda_h(iv),
     $        (fbeam(iv)*uavg_h(lu,iv),lu=lu_start,nlyr+1)
      ENDDO
*
 901  FORMAT(50(1x,f10.3))
 902  FORMAT(1x,f10.3,50(1x,1pe12.4))
      STOP
      END

