      PROGRAM uvspec_inp
*
*BOD
*DESCRIPTION
* 
*INPUT
**albedo
*     real 
*     The surface albedo. May not be combined with albedo_file.
**albedo_file
*     character string
*     The filename, including the path, of the file holding the 
*     wavelength dependent surface albedo values. May not be combined
*     with albedo.
**angstrom
*     two real, alpha and beta
*     Give Angstrom aerosol coefficients alpha and beta. The aerosol
*     optical depth will be scaled to the optical depth calculated
*     using the Angstrom formula.
**atmosphere_file
*     character string
*     The filename, including the path, of the file holding the 
*     atmospheric profiles. Must be enclosed in single quotes.
**julian_day
*     real 
*     Julian day of the year. Used to calculate Earth-Sun distance.
**ozone_column
*     real 
*     The ozone profile is scaled to ozone_column given in DU
*     (Dobson units). If ozone_column is negative, the profile is
*     used as is.
**solar_file
*     character string
*     The filename, including the path, of the file holding the 
*     extraterrestrial spectrum. Must be enclosed in single quotes.
**sza
*     real 
*     The solar zenith angle. If sza is less then 0.0,  sza_file
*     must be given.
**sza_file
*     character string
*     The filename, including the path, of the file holding the 
*     wavelength dependent solar zenith angle values. May not be 
*     combined with sza.
**wvn
*     two reals
*     The first and last wavelengths to be considered.
**zout
*     integer and reals on seperate lines
*     The number of altitudes at which output is given and the 
*     altitudes.      
*OUTPUT
**flup_h
*     real array(zout,lambda_h)
*     The diffuse up flux. See disort documentation.
**lambda_h
*     real array(*)
*     The wavelength grid.
**nlambda_h
*     integer
*     The number of wavelength points.
**rfldir_h
*     real array(zout,lambda_h)
*     The direct flux. See disort documentation.
**rfldn_h
*     real array(zout,lambda_h)
*     The diffuse downward flux. See disort documentation.
**uavg_h
*     real array(zout,lambda_h)
*     The mean intensity. Proportional to the actinic flux.
*     See disort documentation.
*FILES
*     consts.inc                Include file
*EOD
*
      IMPLICIT LOGICAL (A-Z)    ! To nearly get strong typing
      INCLUDE '../src_f/param.inc'
*
* Internal variables
*     
      CHARACTER*100 line
      CHARACTER*100 filenm_aer, filenm_alb, filenm_atm,
     $     filenm_solar, filenm_sza, file_path, filenm_wc
*
      INTEGER atm_type, error, haze, ia, ib, iaea, iaeb, 
     $     iafa, iafb, iala, ialb, ipaa, ipab, isfa, isfb,
     $     isza, iszb, iwca, iwcb,
     $     lu, nlambda, nzout, season, vulcan
      REAL albedo, alpha, beta, day_of_year, flup(maxulv, maxwvn_h),
     $     lambda(maxwvn_h), ozone_column, 
     $     rfldir(maxulv, maxwvn_h), rfldn(maxulv, maxwvn_h), 
     $     sza, wvn_b, wvn_l, uavg(maxulv, maxwvn_h),
     $     uavgdn(maxulv, maxwvn_h), uavgso(maxulv, maxwvn_h),
     $     uavgup(maxulv, maxwvn_h), visib, zout(maxulv)
*
      INTEGER iv, lu_start
*
      INCLUDE '../src_f/consts.inc'
*
* Initialization
*
      albedo       = -1.0
      alpha        = 0.0
      atm_type     = 1
      beta         = 0.0
      ozone_column = -1.0        ! Default, do nothing
      sza          = -1.0
      wvn_b        = -1.0
      wvn_l        = -1.0
      haze         = 0
      vulcan       = 0
      season       = 0
      visib        = 0.0
      iaea         = 0
      iaeb         = 0
      iafa         = 0
      iafb         = 0
      iala         = 0
      ialb         = 0
      ipaa         = 0
      ipab         = 0
      isfa         = 0
      isfb         = 0
      isza         = 0
      iszb         = 0
      iwca         = 0
      iwcb         = 0
      nzout        = 0
      filenm_wc    =  'no_file'
      CALL no_blanks_indices(filenm_wc, iwca, iwcb)
*
      error = 0
      DO WHILE ( .TRUE. ) 
         READ(*,'(A)',END=98 ) line
*
* Process all the no-comment lines
*
         IF ( line(:1) .NE. comcha ) THEN
            CALL no_blanks_indices(line, ia, ib)
*
            IF ( line(ia:ib) .EQ. 'aerosol_tau_file' ) THEN
               READ(*,*) filenm_aer
               CALL no_blanks_indices(filenm_aer, iaea, iaeb)
            ELSE IF ( line(ia:ib) .EQ. 'aerosol_type' ) THEN
               READ(*,*) haze, vulcan, season, visib
            ELSE IF ( line(ia:ib) .EQ. 'albedo' ) THEN
               READ(*,*) albedo
            ELSE IF ( line(ia:ib) .EQ. 'albedo_file' ) THEN
               READ(*,*) filenm_alb
               CALL no_blanks_indices(filenm_alb, iala, ialb)
            ELSE IF ( line(ia:ib) .EQ. 'angstrom' ) THEN
               READ(*,*) alpha, beta
            ELSE IF ( line(ia:ib) .EQ. 'atmosphere_file' ) THEN
               READ(*,*) atm_type, filenm_atm
               CALL no_blanks_indices(filenm_atm, iafa, iafb)
            ELSE IF ( line(ia:ib) .EQ. 'data_files_path' ) THEN
               READ(*,*) file_path
               CALL no_blanks_indices(file_path, ipaa, ipab)
            ELSE IF ( line(ia:ib) .EQ. 'solar_file' ) THEN
               READ(*,*) filenm_solar
               CALL no_blanks_indices(filenm_solar, isfa, isfb)
            ELSE IF ( line(ia:ib) .EQ. 'julian_day' ) THEN
               READ(*,*) day_of_year
            ELSE IF ( line(ia:ib) .EQ. 'ozone_column' ) THEN
               READ(*,*) ozone_column
            ELSE IF ( line(ia:ib) .EQ. 'sza' ) THEN
               READ(*,*) sza
            ELSE IF ( line(ia:ib) .EQ. 'sza_file' ) THEN
               READ(*,*) filenm_sza
               CALL no_blanks_indices(filenm_sza, isza, iszb)
            ELSE IF ( line(ia:ib) .EQ. 'wc_file' ) THEN
               READ(*,*) filenm_wc
               CALL no_blanks_indices(filenm_wc, iwca, iwcb)
            ELSE IF ( line(ia:ib) .EQ. 'wvn' ) THEN
               READ(*,*) wvn_b, wvn_l
            ELSE IF ( line(ia:ib) .EQ. 'zout' ) THEN
               READ(*,*) nzout
               READ(*,*) (zout(lu),lu=1, nzout)
            ELSE
               WRITE(*,*)'Unknown input to uvspec, aborting'
               WRITE(*,*) line
               error = 1
            ENDIF 
         ENDIF 
      ENDDO
 98   CONTINUE
*
* Some checking on input
*
      IF ( iafa .EQ. iafb ) THEN
         WRITE(*,*) 'No atmosphere file specified, aborting.'
         error = 1
      ELSE IF ( (iala .EQ. ialb) .AND. albedo .LT. 0.0 ) THEN
         WRITE(*,*) 'No surface albedo information, aborting.'
         error = 1
      ELSE IF ( isfa .EQ. isfb ) THEN
         WRITE(*,*) 'No extraterrestial solar spectrum file ',
     $        'specified, aborting.'
         error = 1
      ELSE IF ( day_of_year .GT. 365 .OR. day_of_year .LT. 0.0 ) THEN
         WRITE(*,*) 'Julian day out of bound, aborting.'
         error = 1
      ELSE IF ( (isza .EQ. iszb) .AND. sza .LT. 0.0 ) THEN
         WRITE(*,*) 'No surface solar zenith angle information ',
     $        'aborting.'
         error = 1
      ELSE IF ( (wvn_b .GT. wvn_l) .OR.
     $        (wvn_b .LT. 0.0) .OR. (wvn_l .LT. 0.0) ) THEN
         WRITE(*,*) 'wvn out of bound, aborting.'
         error = 1
      ENDIF
      IF ( error .NE. 0 ) STOP
      IF ( nzout .GT. maxulv ) THEN
         WRITE(*,*)'nzout .GT. maxulv', nzout, maxulv
      ENDIF
*
* Then the work
*
      CALL uvspec( albedo, alpha, atm_type, beta, day_of_year, 
     $     file_path(ipaa:ipab),
     $     filenm_aer(iaea:iaeb), filenm_alb(iala:ialb), 
     $     filenm_atm(iafa:iafb), filenm_solar(isfa:isfb),
     $     filenm_sza(isza:iszb), filenm_wc(iwca:iwcb),
     $     flup, haze, lambda, nlambda, nzout,
     $     ozone_column, rfldir, rfldn, season, sza,
     $     uavg, uavgdn, uavgso,  uavgup, visib, vulcan,
     $     wvn_b, wvn_l, zout )
*
*      lu_start = 1
*      WRITE(*,901) (zout(lu),lu=lu_start,nzout)
*      DO iv = 1, nlambda
*         WRITE(*,902) lambda(iv),
*     $        (rfldir(lu,iv), rfldn(lu,iv), flup(lu,iv),
*     $        uavgso(lu,iv), uavgdn(lu,iv), uavgup(lu,iv),
*     $        lu=nzout,lu_start,-1)
*      ENDDO
*
* 901  FORMAT(11x, 50(2x,('<'),18('-'),f10.3,20('-'),('>')))
 901  FORMAT(11x, 50(21x,f10.3,22x))
 902  FORMAT(1x,f10.3,50(1x,1pe12.4))
*
* Read cross sections and quantum yields
*
      
*
      STOP
      END
*
