      SUBROUTINE read_atm(dens, filenm, ndens, nlev, press,
     $     temper, zd, type)
*
*BOD
*DESCRIPTION
* Reads in the afgl model atmospheres in the libRadtran/data/atmmod
* directory.
*USAGE
**FORTRAN
*     CALL read_atm((dens, filenm, nlev, press, temper, zd)
**PERL
*INPUT
**filenm
*     character
*     File name of file holding atmospheric profile data, 
*     must contain NO BLANKS.
**ndens
*     integer
*     Number of densities in the model atmosphere. See dens.
*OUTPUT
**dens
*     real array(levels,species)
*     The density of ndens species at nlev levels. The species index
*     corresponds to the following trace gases for afgls*.dat files:
***1   Air
***2   ozone (o3)
***3   oxygen (o2)
***4   water vapor (h2o)
***5   carbon dioxide (co2)
***6   nitrogen dioxide (no2)
**nlev
*     integer
*     Number of levels in the model atmosphere. Temperature, pressure,
*     altitude and density are returned at nlev levels.
**press
*     real array(0:*)
*     The pressure in mb at levels. 
**temper
*     real array(0:*)
*     The temperature in Kelvin at levels. 
**zd
*     real array(0:*)
*     The altitude in km for levels. 
*ERROR_MESSAGES
*ROUTINES_CALLED
*FILES
*     consts.inc               Include file
*     param.inc                Include file
*REFERENCES
**      Anderson, G. P., S. A. Clough, F. X. Kneizys, 
*         J. H. Chetwynd and E. P. Shettle, AFGL Atmospheric Constituent
*         Profiles (0-120 km), Rep. AFGL-TR-86-0110, AFGL (OPI),
*         Hanscom AFB, MA 01736, 1987.
*EOD
*
      IMPLICIT LOGICAL (A-Z)    ! To nearly get strong typing
      INCLUDE '../src_f/param.inc'
*
      CHARACTER*(*) filenm
      INTEGER error, ndens, nlev, type
      REAL dens(0:maxcly,*), press(0:maxcly), temper(0:maxcly),
     $     zd(0:maxcly)
*
* Internal variables
*
      CHARACTER*100 line
      INTEGER ios, is, iunit
*
      INCLUDE '../src_f/consts.inc'
*
      OPEN(UNIT=iunit,FILE=filenm,STATUS='OLD',
     $     FORM='FORMATTED',ERR=99, IOSTAT=ios)
      GOTO 991
 99   CONTINUE
      WRITE(*,'(A,A)') 'Error during read of file: ', filenm
      WRITE(*,'(A, i4)') 'IOSTAT: ', ios
      error = 1
      RETURN
 991  CONTINUE
*     
      nlev = 0
      DO WHILE ( .TRUE. ) 
         READ(iunit,'(A)',END=98 ) line
*
* Read all the no-comment lines
*
         IF ( line(:1) .NE. comcha ) THEN
            IF (type.EQ.1) THEN
               READ(line,*) zd(nlev), press(nlev), temper(nlev),
     $              (dens(nlev, is),is=1, ndens)
            ELSE IF (type.EQ.2) THEN
               READ(line,*) zd(nlev), press(nlev), temper(nlev),
     $              dens(nlev, iair), dens(nlev, io3)
*               write(*,*) zd(nlev), press(nlev), temper(nlev),
*     $              dens(nlev, iair), dens(nlev, io3)
            ENDIF
            nlev = nlev + 1
         ENDIF
      ENDDO
 98   CONTINUE
      CLOSE(iunit)      
*     
      RETURN
      END
