      SUBROUTINE uvspec(albedo, alpha, atm_type,beta, day_of_year, 
     $     file_path, filenm_aer, filenm_alb,
     $     filenm_atm, filenm_solar, filenm_sza, filenm_wc, 
     $     flup_h, haze, lambda_h, nlambda_h, nzout,
     $     ozone_column, rfldir_h, rfldn_h,
     $     season, sza, uavg_h, uavgdn_h, uavgso_h, uavgup_h, 
     $     visib, vulcan, wvn_b, wvn_l, zout )
*
*BOD
*INPUT
**albedo
*     real 
*     The surface albedo. If albedo is less then 0.0, filenm_alb
*     must be given.
**alpha
*     real
*     Angstrom alpha coefficient.
**beta
*     real
*     Angstrom beta coefficient.
**day_of_year
*     real 
*     Julian day of the year. Used to calculate Earth-Sun distance.
**filenm_aer
*     character string
*     The filename, including the path, of the file holding 
*     aerosol optical depth profile. Total column will be scaled
*     by Angstrom parameters.
**filenm_alb
*     character string
*     The filename, including the path, of the file holding the 
*     wavelength dependent surface albedo values. May not be combined
*     with albedo.
**filenm_atm
*     character string
*     The filename, including the path, of the file holding the 
*     atmospheric profiles.
**filenm_solar
*     character string
*     The filename, including the path, of the file holding the 
*     extraterrestrial spectrum. 
**filenm_sza
*     character string
*     The filename, including the path, of the file holding the 
*     wavelength dependent solar zenith angle values. May not be 
*     combined with sza.
**nzout
*     integer 
*     The number of output altitudes.
**ozone_column
*     real 
*     The ozone profile is scaled to ozone_column given in DU
*     (Dobson units). If ozone_column is negative, the profile is
*     used as is.
**sza
*     real 
*     The solar zenith angle If sza is less then 0.0, filenm_sza 
*     must be given.
**wvn_b, wvn_l
*     two reals
*     The first, wvn_b, and last, wvn_l, wavelengths to be considered.
**zout
*     real array(levels)
*     The output altitudes (km).
*FILES
*     param.inc                Include file
*OUTPUT
**flup_h
*     real array(zout,lambda_h)
*     The diffuse up flux. See disort documentation.
**lambda_h
*     real array(*)
*     The wavelength grid.
**nlambda_h
*     integer
*     The number of wavelength points.
**rfldir_h
*     real array(zout,lambda_h)
*     The direct flux. See disort documentation.
**rfldn_h
*     real array(zout,lambda_h)
*     The diffuse downward flux. See disort documentation.
**uavg_h
*     real array(zout,lambda_h)
*     The mean intensity. Proportional to the actinic flux.
*     See disort documentation.
*EOD
*
      IMPLICIT LOGICAL (A-Z)    ! To nearly get strong typing
      INCLUDE '../src_f/param.inc'
*
*
*
      CHARACTER*(*) filenm_aer, filenm_alb, filenm_atm, 
     $     filenm_solar, filenm_sza, file_path, filenm_wc
      INTEGER atm_type, haze, nlambda_h, nzout, season, vulcan
      REAL albedo, alpha, beta, day_of_year,  
     $     flup_h(maxulv, maxwvn_h),
     $     lambda_h(maxwvn_h), ozone_column, 
     $     rfldir_h(maxulv, maxwvn_h), rfldn_h(maxulv, maxwvn_h), 
     $     sza, wvn_b, wvn_l, uavg_h(maxulv, maxwvn_h),
     $     uavgdn_h(maxulv, maxwvn_h), uavgso_h(maxulv, maxwvn_h), 
     $     uavgup_h(maxulv, maxwvn_h), visib, zout(*)
*
* Internal variables
*
      INTEGER ndens, nlambda_t, nlev, nlyr, ny
      LOGICAL newsiz
      REAL aer_dtau(maxcly,maxwvn), aer_gg(maxcly,maxwvn),
     $     aer_pro(0:maxcly), aer_ssa(maxcly,maxwvn),
     $     albedo_t(maxwvn), dens(0:maxcly,maxden), 
     $     ice_dtau(maxcly,maxwvn), ice_gg(maxcly,maxwvn), 
     $     ice_ssa(maxcly,maxwvn), lambda_alb(maxwvn), 
     $     lambda_t(maxwvn), lambda_sza(maxwvn), 
     $     press(0:maxcly), temper(0:maxcly),
     $     sza_t(maxwvn), wc_dtau(maxcly,maxwvn),
     $     wc_gg(maxcly,maxwvn), wc_ssa(maxcly,maxwvn),
     $     wccon(0:maxcly), wceffr(0:maxcly), wczd(0:maxcly),
     $     zd(0:maxcly)
      REAL dfdt(maxulv, maxwvn), flup(maxulv, maxwvn),
     $     rfldir(maxulv, maxwvn), rfldn(maxulv, maxwvn), 
     $     uavg(maxulv, maxwvn), uavgdn(maxulv, maxwvn),
     $     uavgso(maxulv, maxwvn), uavgup(maxulv, maxwvn)
*
* Aerosol variables
*
      REAL aerabs(mxawvn,0:maxcly), aerext(mxawvn,0:maxcly), 
     $     aersym(mxawvn,0:maxcly), aerwvn(mxawvn), rh(0:maxcly)
*
* High resolution variables
*
      REAL fbeam(maxwvn_h)
*
      REAL eccentricity
*
*  Variables
*
      INTEGER error, ifna, ifnb, iv, lc, lu, lu_start, nalb, 
     $     nsza, nx, nywc
      REAL array(maxarray_x, maxarray_y), ecc_fact,
     $     wvn_step_h,
     $     wvn_step_t
      REAL dummy(maxwvn_h)
*
* INPUT
*
*
* END INPUT
*
*      WRITE(*,*) "ALB  : ", albedo
*      WRITE(*,*) "DAY  : ", day_of_year
*      WRITE(*,*) "OZONE: ", ozone_column
*      WRITE(*,*) "ALB  : ", filenm_alb
*      WRITE(*,*) "ATM  : ", filenm_atm
*      WRITE(*,*) "SOLAR: ", filenm_solar 
*      WRITE(*,*) "SZA  : ", filenm_sza
*      WRITE(*,*) "SZA  : ", sza
*      WRITE(*,*) "WVN  : ", wvn_b, wvn_l
*      WRITE(*,*) "AERO  : ", haze, season, vulcan ,visib
*      STOP
*
* Set wavelength grid for the transmittance calculation
*
c$$$      iv = 1
c$$$      lambda_t(iv) = wvn_b 
c$$$      DO WHILE ( lambda_t(iv) .LE. wvn_l ) 
c$$$         IF (lambda_t(iv) .LT. 350.0 ) THEN
c$$$            wvn_step_t  = 0.5
c$$$         ELSE
c$$$            wvn_step_t  = 1.0
c$$$         ENDIF
c$$$         iv = iv + 1
c$$$*         lambda_t(iv) = wvn_b + float(iv-1)*wvn_step_t
c$$$         lambda_t(iv) = lambda_t(iv-1) + wvn_step_t
c$$$*         WRITE(*,*)"lambda",iv, lambda_t(iv), float(iv-1)*wvn_step_t, 
c$$$*     $        wvn_b, wvn_l
c$$$      ENDDO
c$$$      nlambda_t   = iv
c$$$      IF ( nlambda_t .GT. maxwvn ) THEN         
c$$$            WRITE(*,*)'uvspec: nlambda_t .GT. maxwvn', nlambda_t,
c$$$     $        maxwvn
c$$$            STOP
c$$$      ENDIF
*
* Albedo stuff
*
      IF ( albedo .LT. 0.0 ) THEN
         nx = 2
         CALL read_array(array, error, maxarray_x, nx, nalb,
     $     filenm_alb)         
         IF ( error .NE. 0 ) THEN
            WRITE(*,*)'read_array reports error:', error
            STOP
         ENDIF
         DO iv = 1, nalb
            lambda_alb(iv) = array(1,iv)
            dummy(iv)   = array(2,iv)
         ENDDO
* Interpolate to transmission grid
         CALL arb_wvn(lambda_t, lambda_alb, nlambda_t, nalb,
     $        1, dummy, albedo_t, 1)
      ELSE
         DO iv = 1, nlambda_t
            albedo_t(iv) = albedo
         ENDDO
      ENDIF
*
* Solar zenith angle stuff
*
      IF ( sza .LT. 0.0 ) THEN
         nx = 2
         CALL read_array(array, error, maxarray_x, nx, nsza,
     $     filenm_sza)         
         IF ( error .NE. 0 ) THEN
            WRITE(*,*)'read_array reports error:', error
            STOP
         ENDIF
         DO iv = 1, nsza
            lambda_sza(iv) = array(1,iv)
            dummy(iv)   = array(2,iv)
         ENDDO
* Interpolate to transmission grid
         CALL arb_wvn(lambda_t, lambda_sza, nlambda_t, nsza,
     $        1, dummy, sza_t, 1 )
      ELSE
         DO iv = 1, nlambda_t
            sza_t(iv) = sza
         ENDDO
      ENDIF
*
* Correction factor for Earth-Sun distance
*
c$$$      ecc_fact = 1.0
c$$$      IF ( day_of_year .GT. 0. ) THEN
c$$$         ecc_fact = eccentricity(day_of_year) 
c$$$      ENDIF
*      WRITE(*,*)"Eccent", ecc_fact
*      STOP
*
* Read extraterrestrial spectrum
*
      nx = 2
      CALL no_blanks_indices(filenm_solar, ifna, ifnb)
      CALL read_array(array, error, maxarray_x, nx, ny,
     $     filenm_solar(ifna:ifnb))
      IF ( error .NE. 0 ) THEN
         WRITE(*,*)'read_array reports error:', error
         STOP
      ENDIF
      nlambda_h = 1
      iv = 1
      DO WHILE ( array(1,iv) .LE. wvn_l  ) 
         IF ( array(1,iv) .GE. wvn_b) THEN
            lambda_h(nlambda_h) = array(1,iv)
            fbeam(nlambda_h)    = array(2,iv)*ecc_fact
*            WRITE(*,*)"UVSPEC",nlambda_h, iv, lambda_h(nlambda_h),
*     $           wvn_l
            nlambda_h           = nlambda_h + 1
         ENDIF
         iv = iv + 1
      ENDDO
      nlambda_h           = nlambda_h - 1
      IF ( nlambda_h .GT. maxwvn_h ) THEN         
            WRITE(*,*)'uvspec: nlambda_h .GT. maxwvn_h', nlambda_h,
     $        maxwvn_h
            STOP
      ENDIF
*
* Read in model atmosphere 
*
c$$$      ndens = 6
c$$$      CALL no_blanks_indices(filenm_atm, ifna, ifnb)
c$$$      CALL read_atm(dens, filenm_atm(ifna:ifnb), ndens, nlev, press,
c$$$     $     temper, zd, atm_type)
c$$$      nlyr = nlev - 1
c$$$      IF ( nlyr .GT. maxcly ) THEN         
c$$$            WRITE(*,*)'uvspec: nlyr .GT. maxcly', nlyr, maxcly
c$$$            STOP
c$$$      ENDIF      
*
* Scale ozone column
*
cakyp
*      DO lc = 0, nlyr
*         WRITE(12,*)"before",lc,zd(lc),dens(lc,io3)
*      ENDDO
cakyp
c$$$      IF ( ozone_column .GE. 0.0 ) THEN
c$$$         CALL scale_ozone(dens(0,io3), nlyr, ozone_column, zd) 
c$$$      ENDIF
cakyp
*      DO lc = 0, nlyr
*         WRITE(12,*)"after",lc,zd(lc),dens(lc,io3)
*      ENDDO
*      STOP
cakyp
*
* Water cloud stuff
*
      IF ( filenm_wc .NE. 'no_file' ) THEN
*
         nx = 3
         CALL read_array(array, error, maxarray_x, nx, nywc,
     $     filenm_wc)
         IF ( error .NE. 0 ) THEN
            WRITE(*,*)'read_array reports error:', error
            STOP
         ELSE IF ( nywc .NE. nlyr+1 ) THEN
            WRITE(*,*)'wc file wrong resolution:', nywc, nlyr+1
            STOP
         ENDIF
         DO lc = 0, nlyr
            wczd(lc)   = array(1,lc+1)
            wccon(lc)  = array(2,lc+1)
            wceffr(lc) = array(3,lc+1)
            IF ( wczd(lc) .NE.  zd(lc) ) THEN
               WRITE(*,*)'wczd different from zd:', wczd(lc), zd(lc)
               STOP               
            ENDIF
         ENDDO
         newsiz = .TRUE.
         DO iv = 1, nlambda_t
            CALL wcloud( lambda_t(iv), newsiz, nlyr, file_path, 
     $           wccon, wceffr, wc_dtau(1,iv), wc_gg(1,iv), 
     $           wc_ssa(1,iv), zd )
         ENDDO
      ENDIF
*     
* Aerosol stuff
*
      IF ( vulcan .GT. 0 ) THEN
*   Get appropriate aerosol profile and interpolate profile
*   to zd grid.
         CALL aer_prof( nlyr, zd, aer_pro, season, vulcan, visib )
*   Get appropriate aerosol optical properties.
         CALL aer_opt( vulcan, aerext, aerabs, aersym, aerwvn, rh,
     $     nlyr, zd, haze )
         CALL aer_wvn( aerwvn, aerabs, aersym, aerext, aer_pro, 
     $     aer_dtau, aer_gg, aer_ssa, nlyr, lambda_t, zd, nlambda_t )
*   Read in aerosol optical depth file if specified
         CALL no_blanks_indices(filenm_aer, ifna, ifnb)
         IF (ifna .LT. ifnb ) THEN
            nx = 2
            CALL read_array(array, error, maxarray_x, nx, ny,
     $           filenm_aer(ifna:ifnb))
            IF ( error .NE. 0 ) THEN
               WRITE(*,*)'read_array reports error:', error
               STOP
            ENDIF
            IF ( ny .NE. nlev ) THEN
               WRITE(*,*)'Aerosol file, ny .NE. nlyr:', ny, nlev
               STOP
            ENDIF
            DO iv = 1, nlambda_t
               DO lc = 1, nlyr
                  aer_dtau(lc,iv) = array(2,lc)
               ENDDO
            ENDDO
         ENDIF
         IF ( beta .NE. 0.0 ) THEN
            CALL scale_aer(aer_dtau, lambda_t, nlambda_t, alpha,
     $           beta, nlyr)
         ENDIF
      ENDIF
*
* Calculate radiation field with fbeam = 1.0, not exactly the same as 
* the transmittance, but no extraterrestrial spectrum involved.
*
      CALL trans(aer_dtau, aer_gg, aer_ssa, albedo_t, dens, 
     $     dfdt, flup, ice_dtau, ice_gg, ice_ssa, lambda_t,
     $     ndens, nlambda_t, nlyr, nzout, file_path, press, rfldir,
     $     rfldn, sza_t, temper, uavg,  uavgdn, uavgso, uavgup,
     $     wc_dtau, wc_gg, wc_ssa, zd, zout)
*
*      lu_start = 1
*      WRITE(*,901) (zout(lu),lu=lu_start,nzout)
*      DO iv = 1, nlambda_t
*         WRITE(*,902) lambda_t(iv), (rfldir(lu,iv),
*     $        rfldn(lu,iv), flup(lu,iv), uavg(lu,iv)
*     $        ,lu=nzout,lu_start,-1)
*         WRITE(*,902) lambda_t(iv), (fbeam(iv)*rfldir(lu,iv),
*     $        fbeam(iv)*rfldn(lu,iv), fbeam(iv)*flup(lu,iv), 
*     $        fbeam(iv)*uavg(lu,iv)
*     $        ,lu=nzout,lu_start,-1)
*      ENDDO
*
* Interpolate to arbitrary wavelength grid using qubic splines.
*
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nzout, flup, flup_h, maxulv)
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nzout, rfldir, rfldir_h, maxulv)
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nzout, rfldn, rfldn_h, maxulv)
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nzout, uavg, uavg_h, maxulv)
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nzout, uavgdn, uavgdn_h, maxulv)
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nzout, uavgso, uavgso_h, maxulv)
      CALL arb_wvn(lambda_h, lambda_t, nlambda_h, nlambda_t,
     $      nzout, uavgup, uavgup_h, maxulv)
*
*      lu_start = 1
*      WRITE(*,901) (zout(lu),lu=lu_start,nzout)
*      DO iv = 1, nlambda_h
*         WRITE(*,902) lambda_h(iv), (rfldir_h(lu,iv),
*     $        rfldn_h(lu,iv), flup_h(lu,iv), uavg_h(lu,iv)
*     $        ,lu=nzout,lu_start,-1)
*         WRITE(*,902) lambda_h(iv), (fbeam(iv)*rfldir_h(lu,iv),
*     $        fbeam(iv)*rfldn_h(lu,iv), fbeam(iv)*flup_h(lu,iv), 
*     $        fbeam(iv)*uavg_h(lu,iv)
*     $        ,lu=nzout,lu_start,-1)
*      ENDDO
*
      DO lu = 1, nzout
         DO iv = 1, nlambda_h
*            WRITE(*,*)"UVSPEC2",lu,iv,lambda_h(iv),fbeam(iv),
*     $           rfldir_h(lu,iv)
            flup_h(lu,iv)   = flup_h(lu,iv) * fbeam(iv)
            rfldir_h(lu,iv) = rfldir_h(lu,iv) * fbeam(iv)
            rfldn_h(lu,iv)  = rfldn_h(lu,iv) * fbeam(iv)
            uavg_h(lu,iv)   = uavg_h(lu,iv) * fbeam(iv)
            uavgdn_h(lu,iv) = uavgdn_h(lu,iv) * fbeam(iv)
            uavgso_h(lu,iv) = uavgso_h(lu,iv) * fbeam(iv)
            uavgup_h(lu,iv) = uavgup_h(lu,iv) * fbeam(iv)
            IF ( flup_h(lu,iv) .LT. 0.0 ) flup_h(lu,iv) = 0.0
            IF ( rfldir_h(lu,iv) .LT. 0.0 ) rfldir_h(lu,iv) = 0.0
            IF ( rfldn_h(lu,iv) .LT. 0.0 ) rfldn_h(lu,iv) = 0.0
            IF ( uavg_h(lu,iv) .LT. 0.0 ) uavg_h(lu,iv) = 0.0
            IF ( uavgdn_h(lu,iv) .LT. 0.0 ) uavgdn_h(lu,iv) = 0.0
            IF ( uavgso_h(lu,iv) .LT. 0.0 ) uavgso_h(lu,iv) = 0.0
            IF ( uavgup_h(lu,iv) .LT. 0.0 ) uavgup_h(lu,iv) = 0.0
         ENDDO
      ENDDO
*
*
*      lu_start = 1
*      WRITE(*,901) (zout(lu),lu=lu_start,nzout)
*      DO iv = 1, nlambda_h
*         WRITE(*,902) lambda_h(iv),
*     $        (fbeam(iv)*uavg_h(lu,iv),lu=nzout,lu_start,-1)
*      ENDDO
*
 901  FORMAT(50(1x,f10.3))
 902  FORMAT(1x,f10.3,50(1x,1pe12.4))
      RETURN
      END

