# sdoc.awk
#                                                                      
# sdoc - Automatic source documentation                      @ti@ 
#                                                                      
# Author:                                                              
#   Bernhard Mayer                                                @au@ 
#   Fraunhofer Institute for Atmospheric Environmental Research,  @ad@ 
#   82467 Garmisch-Partenkirchen,                                 @ad@ 
#   Germany                                                       @ad@ 
#                                                                      
# --------------------------------------------------------------------
# Copyright (C) 1997 Bernhard Mayer                               @ci@ 
#                                                                      
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 1, or (at your option)  
# any later version.                                                   
#                                                                      
# This program is distributed in the hope that it will be useful,      
# but WITHOUT ANY WARRANTY; without even the implied warranty of       
# MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         
# GNU General Public License for more details.                         
#                                                                      
# To obtain a copy of the GNU General Public License write to the      
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   
# USA.                                                            @ic@ 
#---------------------------------------------------------------------


# @strong{sdoc} is a tool for automatic documentation of source code.      @oc@
# It may be used for documentation of C, Fortran and AWK code
# if the comments follow some simple rules. Extension to other
# languages should be straightforward. The generated documentation
# is written in texinfo. Comments which are to be included into
# the documentation may therefore contain any valid texinfo 
# command. An example for a sdoc generated file is the document which
# you are currently reading. 
#
# sdoc is a awk script which was tested with GNU awk 3.0.0
# (version 2.15 will @strong{NOT} work!).
#
# @strong{Usage:}
# @example
# gawk -f sdoc.awk -vlanguage="lang" -vlevel="lev" 
#                  -vsecname="name" -voutput="out"
#                  -vnumbering="num" -vpagebreak="page"
#                  <source file> > <texinfo file> 
# @end example
# where "lang" is either "c", "fortran", or "awk", @nl@
#       "lev"  is either "document", "chapter", "section", or "subsection" @nl@
#       "name" is the name of the chapter, section or subsection @nl@
#       "out"  is either "latex" or "texinfo" @nl@
#       "num"  is either "on" or "off"  @nl@
#       "page" is either "on" or "off"
#
# "lang" defaults to "c", "lev" to "document", "name" to the filename
# without extension, "out" to "texinfo", "num" to "on", and "page" to "off" 
#
# @strong{level:} "document" means that a complete document will be created
# while "chapter", "section", or "subsection" will create only 
# part of a document.
#
# @strong{output:} if latex is chosen, the output of sdoc.awk has
# to be piped through 'gawk -f texinfo2latex.awk'
#
# @strong{numbering:} "on"   numbered sections,
#                     "off"  unnumbered sections
#
# @strong{pagebreak:} "on"   pagebreak after each section,
#                     "off"  no pagebreak after each section
#
# There are two ways of including more than one source files
# in a documentation:
# @enumerate
# @item 
# @example
# gawk -f sdoc.awk -vlevel="section" -vsecname="allfuncs" 
#                  source1.c source2.c ... > source.texi
# @end example
# will create @strong{one} section "allfuncs" containing the documentation 
# for all files.
#
# @item 
# @example
# gawk -f sdoc.awk source1.c  >  source1.texi
# gawk -f sdoc.awk source2.c  >  source2.texi
# ...
# @end example
# will create separate sections for all source files, named "source1", "source2", ...
# which must be included separately.
# @end enumerate
#
# For more examples have a look at the "makefile" which is
# included in the distribution. 
#
# The documentation is generated according to the following rules:        
#
# @enumerate
# @item The generated texinfo file has four chapters: 
#   @enumerate
#   @item Copying     (copying conditions)
#   @item Overview    (overview over the source package)
#   @item Definitions (definitions of variables, macros, ...)
#   @item Functions   (definitions of functions)
#   @end enumerate
#
# @item sdoc commands are generally written as \@ xy \@ where x and 
#   y are ASCII characters.
#
# @item If xy is the command for 'begin something', 
#   yx is the command for 'end something'.
#
# @item Comment inclusion starts with the comment where a 
#   xy = 'oc', 'dc', or 'fc' was found and end with the comment
#   where a xy = 'co', 'cd', or 'cf' was found. Commands may
#   be placed anywhere within a comment. Comments are NOT 
#   restricted to single lines!
#
# @item Source inclusion starts with the first line after 
#   the comment where a xy = 'os', 'ds', or 'fs' was found 
#   and ends with the last line before the comment
#   where a xy = 'so', 'sd', or 'sf' was found.
#
# @item Command names:
#   @table @asis
#   @item x='c'  
#           'Copying' section
#
#   @item x='o'  
#           'Overview' section
#
#   @item x='d'
#           'Definitions' section
#
#   @item x='f':  
#           'Functions' section
#   @end table
#
#
#   @table @asis
#   @item y='c'
#           Comment inclusion 
#           (exception: in the 'Copying' section 'i' is used instead)
#
#   @item y='s'
#           Source inclusion
#    
#   @item y='i' 
#           Something special, e.g. automatic inclusion 
#           of a function.  
#   @end table 
#
# @item Special commands: 
#   @table @asis
#   @item xy = 'ti'   
#           Document title
#   @item xy = 'au'   
#           Author name
#   @item xy = 'ad'   
#           Author address
#   @item xy = 'yr'
#           Year of copyright notice
#   @item xy = 'nl'
#           Newline character
#   @end table
#
# @end enumerate
#
#
# @strong{Examples} (example files are included in the package)
# @itemize @bullet
# @item C source:
#   @example
#   gawk -f sdoc.awk ascii.h ascii.c > ascii.texi 
#   @end example
# @item C source, more complicated:
#   @example
#   make uvbtexinfo
#   @end example
# will create a texinfo documentation for the uvb package which 
# consists of two libraries, ASCII and NUMERIC.
#   @example
#   make uvblatex
#   @end example 
# will create a LaTeX file with the same contents.
#
# @item Fortran:
#   @example
#   gawk -f sdoc.awk -vlanguage="fortran" absdep.f > absdep.texi 
#   @end example
# @item awk:
#   @example
#   gawk -f sdoc.awk -vlanguage="awk" sdoc.awk > sdoc.texi 
#   @end example
# or
#   @example
#   make sdoc 
#   @end example
# will create the documentation which you are currently reading.
# @end itemize
# @co@


# Remove all command strings from a comment  @fi@ @if@
function cleanup(string)
{
  # clear string, if only ****
  match(string, /\*+/);
  if (RLENGTH==length(string)) {
    string="";
    return string;
  }

  # 'copying' comment inclusion
  gsub("@ci@","",string);
  gsub("@ic@","",string);


  # 'overview' source inclusion
  gsub("@os@","",string);
  gsub("@so@","",string);

  # 'overview' comment inclusion
  gsub("@oc@","",string);
  gsub("@co@","",string);
  

  # 'functions' source inclusion
  gsub("@fc@","",string);
  gsub("@cf@","",string);

  # 'functions' comment inclusion
  gsub("@fs@","",string);
  gsub("@sf@","",string);

  # 'functions' inclusion
  gsub("@fi@","",string);
  gsub("@if@","",string);

  # 'definitions' source inclusion
  gsub("@ds@","",string);
  gsub("@sd@","",string);

  # 'definitions' comment inclusion
  gsub("@dc@","",string);
  gsub("@cd@","",string);


  # newline character
  gsub("@nl@","\n",string);
  

  return string;
}


# Print a header for the texinfo output @fi@ @if@
function print_texinfo_header(filename, infoname, year, author, address, title)
{
  if (level=="document") {
    print "\\input texinfo  @c -*-texinfo-*-";
    print "@c %**start of header";
    print "@setfilename "infoname;
    print "@settitle   "title;
    print "@c %**end of header";
    print "";
    print "";
    print "@c @setchapternewpage odd";
    print "";
    print "@ifinfo";
    print title;
    print "";
#    print "Copyright @copyright{} "year" "author;
    print "@end ifinfo";
    print "";
    print "";
    print "@titlepage";
    print "@title "title;
    print "@author "author;
    print "";
    print "";
    print "@c  The following two commands";
    print "@c  start the copyright page.";
    print "@page";
    print "@vskip 0pt plus 1filll";
#    print "Copyright @copyright{} "year" "author;
    print "";
    print "@end titlepage";
    print "";
    print "@ifinfo";
    print "@node Top, Copying, (dir), (dir)";
    print "@top sdoc";
    print "@end ifinfo";
    print "";
    print "@menu";
    print "* Copying::          Your rights and freedoms.";
    print "* Overview::         Overview of the library.";
    print "* Definitions::      Type and macro definitions.";
    print "* Functions::        Function definitions.";
    print "* Functions index::  Index of functions";
    print "@end menu";
    print "";
  } 
}


# Print a header for the latex output @fi@ @if@
function print_latex_header(filename, infoname, year, author, address, title)
{
  if (level=="document") {
    print "\\documentstyle[12pt]{report}";
    print "\\begin{document}";
    print "\\begin{titlepage}";
#    print "\\title{"title"}";
#    print "\\author{"author"}";
    print "\\end{titlepage}";
    print "\\maketitle";

    # ??? copyright note not considered up to now
#    # print "Copyright @copyright{} "year" "author;
    # print "\\maketitle";
    # print "@c  The following two commands";
    # print "@c  start the copyright page.";
    # print "@page";
    # print "@vskip 0pt plus 1filll";
#    # print "Copyright @copyright{} "year" "author;

    print "\\newpage";
    print "\\tableofcontents";
    print "\\newpage";
  }
}


# Print node and section  @fi@ @if@
function print_texinfo_node(nodename, title)
{

  if (level=="chapter") {
    print "@node "nodename;
#    print chapter_header" "title;
    print "";
  }


  if (level=="section") {
    print "@node "nodename;
#    print section_header" "title;
    print "";
  }

  if (level=="subsection") {
    print "@node "nodename;
#    print subsection_header" "title;
    print "";
  }
}

# Print node and section  @fi@ @if@
function print_texinfo_section(nodename, title, string)
{

  if (level=="chapter") {
    print "@node "nodename;
#    print chapter_header" "title;
    print "";
  }


  if (level=="section") {
    print "@node "nodename;
#    print section_header" "title;
    print "";
  }

  if (level=="subsection") {
    print "@node "nodename;
#    print subsection_header" "title;
    print "";
  }
  print string;
  print "";

}

# Print the texinfo 'Copying' chapter  @fi@ @if@
function print_texinfo_copying(string)
{
  if (level=="document")  {
    print "@node Copying, Overview, Top, Top";
    print chapter_header" Copying";

    print string;
    print "";
  
    if (pagebreak=="on")
      print "@page";
  }
}


# Print the texinfo 'Overview' chapter  @fi@ @if@
function print_texinfo_overview(string)
{
  if (level=="document")  {
    print "@node Overview, Definitions, Copying, Top";
    print chapter_header" Overview";
  }

  if (level=="chapter")  {
#    print "@node "secname".Overview, "secname".Definitions, "secname", "secname;
    print section_header" Overview";
  }

  if (level=="section")  {
#    print "@node "secname".Overview, "secname".Definitions, "secname", "secname;
    print subsection_header" Overview";
  }

  if (level=="subsection")  {
#    print "@node "secname".Overview, "secname".Definitions, "secname", "secname;
    print subsubsection_header" Overview";
  }

  print string;
  print "";

  if (pagebreak=="on")
    print "@page";
}


# Print the texinfo 'Definitions' chapter  @fi@ @if@
function print_texinfo_definitions(string)
{
  if (level=="document")  {
    print "@node Definitions, Functions, Overview, Top";
    print chapter_header" Definitions";
  }

  if (level=="chapter")  {
#    print "@node "secname".Definitions, "secname".Functions, "secname".Overview, "secname;
    print section_header" Definitions";
  }

  if (level=="section")  {
#    print "@node "secname".Definitions, "secname".Functions, "secname".Overview, "secname;
    print subsection_header" Definitions";
  }

  if (level=="subsection")  {
#    print "@node "secname".Definitions, "secname".Functions, "secname".Overview, "secname;
    print subsubsection_header" Definitions";
  }

  print string;
  print "";

  if (pagebreak=="on")
    print "@page";
}


# Print the texinfo 'Functions' chapter  @fi@ @if@
function print_texinfo_functions(string)
{
  if (level=="document")  {
    print "@node Functions, Functions index, Definitions, Top";
    print "@chapter Functions";
  }

  if (level=="chapter")  {
#    print "@node "secname".Functions, , "secname".Definitions, "secname;
    print section_header" Functions";
  }

  if (level=="section")  {
#    print "@node "secname".Functions, , "secname".Definitions, "secname;
    print subsection_header" Functions";
  }

  if (level=="subsection")  {
#    print "@node "secname".Functions, , "secname".Definitions, "secname;
    print subsubsection_header" Functions";
  }

  print string;
  print "";

  if (pagebreak=="on")
    print "@page";
}


# print a submenu (only if level=="section" or "subsection") @fi@ @if@
function print_texinfo_submenu()
{
#  if(level=="chapter" || level=="section" || level=="subsection")  {
#    print "@menu";
#    print "* "secname".Overview::         Overview of the module.";
#    print "* "secname".Definitions::      Definitions.";
#    print "* "secname".Functions::        Functions.";
#    print "@end menu";
#  }
}


# Print the end of the texinfo output (table of contents, index of functions, ...) @fi@ @if@
function print_texinfo_end()
{
  if (level=="document") {
    print "@node Functions index, , Functions, Top";
    print "@unnumbered Index of functions";
    print "@printindex fn";
    print "@contents";
    print "";
    print "@bye";
  }
}


# Print the end of the latex output @fi@ @if@
function print_latex_end()
{
  if (level=="document") {
    print "\\end{document}";
  }
}




BEGIN { 
  language=tolower(language);
  level=tolower(level);
  output=tolower(output);
  numbering=tolower(numbering);
  pagebreak=tolower(pagebreak);

  # default language: C
#  if(language!="fortran" && language!="awk")
#    language="c";

  # default level: Document
  if(level!="document" && level!="chapter" && level!="section" && level!="subsection")
    level="document";

  # default output: texinfo
  if(output!="texinfo" && output!="latex")
    output="texinfo";

  #default: numbering="on"
  if(numbering!="on" && numbering!="off")
    numbering="on";

  #default: pagebreak="off"
  if(pagebreak!="on" && pagebreak!="off")
    pagebreak="off";
  

  # C comments start with "/*" and end with "*/" 
  if(language=="c") {
    comment_start="\\/\\*";
    comment_end  ="\\*\\/";
  }

  # Fortran comments start with "*" and end with "\n" 
  if(language=="fortran") {
    comment_start="\\*"; 
    comment_end  ="\\n";
  }

  # awk comments start with "#" and end with "\n" 
  if(language=="awk") {
    comment_start="#"; 
    comment_end  ="\\n";
  }


  # section headings 
  if (numbering=="on") {
    chapter_header       = "@chapter";
    section_header       = "@section";
    subsection_header    = "@subsection";
    subsubsection_header = "@subsubsection";
  }
  else {
    chapter_header       = "@unnumbered";
    section_header       = "@unnumberedsec";
    subsection_header    = "@unnumberedsubsec";
    subsubsection_header = "@unnumberedsubsubsec";
  }


  # how to indicate "verbatim" in Texinfo
  source_start_symbol = "@example";
  source_end_symbol   = "@end example";

  RS=comment_start; 
  
  # copying section
  copying_comment_inclusion=0;     # flag indicating inclusion of comments

  # overview section
  overview_comment_inclusion=0;    # flag indicating inclusion of comments
  overview_source_inclusion=0;     # flag indicating inclusion of source
  first_overview_source_line=0;    # flag indicating start of source inclusion

  # definitions section
  definition_comment_inclusion=0;    # flag indicating inclusion of comments
  definition_source_inclusion=0;     # flag indicating inclusion of source
  first_definition_source_line=0;    # flag indicating start of source inclusion

  # function section
  function_comment_inclusion=0;    # flag indicating inclusion of comments
  function_source_inclusion=0;     # flag indicating inclusion of source
  function_inclusion=0;            # flag indicating inclusion of function
  function_description=0;          # flag indicating inclusion of function description
  first_function_source_line=0;    # flag indicating start of source inclusion

  copying="";
  overview="";
  definitions="";
  functions="";
  func_name="";
  func_description="";
  author="";
  address="";
  year="";
  title="";
}


# if (record separator is comment_start), which means, we are in the source
RS==comment_start {

  if (first_overview_source_line==1) {
    overview = overview"\n"source_start_symbol;
    first_overview_source_line=0;
  }

  if (overview_source_inclusion)
    overview = overview"\n"cleanup($0);
  


  if (first_definition_source_line==1) {
    definitions = definitions"\n"source_start_symbol;
    first_definition_source_line=0;
  }

  if (definition_source_inclusion)
    definitions = definitions"\n"cleanup($0);
  


  if (first_function_source_line==1) {
    functions = functions"\n"source_start_symbol;
    first_function_source_line=0;
  }

  if (function_source_inclusion)
    functions = functions"\n"cleanup($0);
  
  if (function_inclusion) {
    if(!match($0, ")"))
      func_name = func_name"\n"cleanup($0);
    else {
      split($0,temp, ")");
      if (func_name="")
	func_name = temp[1]")\n";
      else 
	func_name = func_name"\n"temp[1]")\n";

      # create single line with single space only
      gsub(/\n/,"",func_name);
      gsub(/[ \t]+/," ",func_name);

      # remove "$" from function definition if fortran
      if(language=="fortran")
	gsub(/\$/, "", func_name);

      functions = functions"\n  @deftypefun "func_name"\n\n"func_description"@end deftypefun\n";
      function_inclusion=0;
      func_name="";
      func_description="";

    }
  }

  RS=comment_end;
  next;
}


# if (record separator is comment_end), which means, we are inside a comment
RS==comment_end {

  # author
  if (gsub("@au@", ""))
    author = author""$0;

  # address
  if (gsub("@ad@", ""))
    address = address""$0;

  # year
  if (gsub("@yr@", ""))
    year = year""$0;

  # title
  if (gsub("@ti@", ""))
    title = title""$0;


  # 'start of overview source inclusion' marker
  if (match($0, "@os@")) {
    overview_source_inclusion=1;
    first_overview_source_line=1;
  }

  # 'end of source inclusion' marker
  if (match($0, "@so@")) {
    overview_source_inclusion=0;
    overview = overview"\n"source_end_symbol;
  }


  # 'start of copying comment inclusion' marker
  if (match($0, "@ci@"))
    copying_comment_inclusion=1;

  # if 'comment inclusion'
  if (copying_comment_inclusion) {

    # 'end of comment inclusion' marker
    if (match($0, "@ic@"))
      copying_comment_inclusion=0;

    copying=copying"\n"cleanup($0);
  }


  # 'start of overview comment inclusion' marker
  if (match($0, "@oc@"))
    overview_comment_inclusion=1;

  # if 'comment inclusion'
  if (overview_comment_inclusion) {

    # 'end of comment inclusion' marker
    if (match($0, "@co@"))
      overview_comment_inclusion=0;

    overview=overview"\n"cleanup($0);
  }




  # 'start of definitions source inclusion' marker
  if (match($0, "@ds@")) {
    definition_source_inclusion=1;
    first_definition_source_line=1;
  }

  # 'end of source inclusion' marker
  if (match($0, "@sd@")) {
    definition_source_inclusion=0;
    definitions = definitions"\n"source_end_symbol;
  }


  # 'start of definition comment inclusion' marker
  if (match($0, "@dc@"))
    definition_comment_inclusion=1;

  # if 'comment inclusion'
  if (definition_comment_inclusion) {

    # 'end of comment inclusion' marker
    if (match($0, "@cd@"))
      definition_comment_inclusion=0;

    definitions = definitions"\n"cleanup($0);
  }




  # 'start of function source inclusion' marker
  if (match($0, "@fs@")) {
    function_source_inclusion=1;
    first_function_source_line=1;
  }

  # 'end of source inclusion' marker
  if (match($0, "@sf@")) {
    function_source_inclusion=0;
    functions = functions"\n"source_end_symbol;
  }


  # 'start of function comment inclusion' marker
  if (match($0, "@fc@"))
    function_comment_inclusion=1;

  # if 'comment inclusion'
  if (function_comment_inclusion) {

    # 'end of comment inclusion' marker
    if (match($0, "@cf@"))
      function_comment_inclusion=0;

    functions = functions"\n"cleanup($0);
  }


  # 'start of function inclusion' marker
  if (match($0, "@fi@")) {
    function_inclusion=1;
    function_description=1;
  }
  
  # if 'function inclusion'
  if (function_description) {

    # 'end of function inclusion' marker
    if (match($0, "@if@"))
      function_description=0;
    
    func_description = func_description"\n"cleanup($0);
  }


  RS=comment_start;
  next;
}


END {
  gsub(/\n\n+/,"\n\n", copying);
  gsub(/\n\n+/,"\n\n", overview);
  gsub(/\n\n+/,"\n\n", definitions);
  gsub(/\n\n+/,"\n\n", functions);

  # create the name for the .info file
  number = split (FILENAME,temp,".");
  if (number>1)
    basename=temp[1];
  else 
    basename=FILENAME;

  for (i=2; i<number; i++)
    basename = basename"."temp[i];
  
  infoname = basename".info";

  if (secname=="")
    secname=basename;  

  if (output=="texinfo") {
    print_texinfo_header(FILENAME, infoname, year, author, address, title);
    print_texinfo_section(title, title, definitions);
#    print_texinfo_submenu();
#    print_texinfo_copying(copying);
#    print_texinfo_overview(overview);
#    print_texinfo_definitions(definitions);
#    print_texinfo_functions(functions);
    print_texinfo_end();
  }

  if (output=="latex") {
    print_latex_header(FILENAME, infoname, year, author, address, title);

    # latex does not like "\"
    gsub(/\\/, "$\\backslash$", copying);
    gsub(/\\/, "$\\backslash$", overview);
    gsub(/\\/, "$\\backslash$", definitions);
    gsub(/\\/, "$\\backslash$", functions);

    print_texinfo_node(title, title);
    print_texinfo_copying(copying);
    print_texinfo_overview(overview);
    print_texinfo_definitions(definitions);
    print_texinfo_functions(functions);
    print_latex_end();
  }
}
