/************************************************************************/
/* linear.c                                                             */
/*                                                                      */
/* Linear interpolation with an interface compatible to spline.c        */
/* (interpolation can be done with calc_spline_values()                 */
/*                                                                      */
/* Author: Bernhard Mayer,                                              */
/*        Fraunhofer Institute for Atmospheric Environmental Research,  */
/*        82467 Garmisch-Partenkirchen,                                 */
/*        Germany                                                       */
/*                                                                      */
/*----------------------------------------------------------------------*/
/* Copyright (C) 1995 Bernhard Mayer                                    */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*----------------------------------------------------------------------*/
/************************************************************************/



#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <malloc.h>

#include "numeric.h"




/************************************************************/
/* Calculate coefficients for linear interpolation;         */
/* memory for coefficients will be allocated automatically! */
/* for compatibility with spline interpolation, four        */
/* coefficients are calculated, but a2[] and a3[] are set   */
/* to zero.                                                 */
/************************************************************/

int linear_coeffc (double *x, double *y, int number, 
		   double **a0, double **a1, double **a2, double **a3)
{
  int i=0;


  /* check if x ascending */
  for (i=0; i<number-1; i++)  {
    if (x[i]>=x[i+1])
      return X_NOT_ASCENDING;
  }

  
  /* allocate memory for coefficients */
  *a0 = (double *) calloc (number, sizeof(double));
  *a1 = (double *) calloc (number, sizeof(double));
  *a2 = (double *) calloc (number, sizeof(double));
  *a3 = (double *) calloc (number, sizeof(double));


  /* calculate coefficients */
  for (i=0; i<number-1; i++)  {
    (*a0)[i] = y[i];
    (*a1)[i] = (y[i+1]-y[i]) / (x[i+1]-x[i]);
  }


  return 0;   /* if o.k. */
}
