      SUBROUTINE rayleigh( crs, error, lambda, n_lambda )
*
*BOD
*DESCRIPTION
* Calculates the Rayleigh scattering cross sections as a function of
* wavelength.
*USAGE
**FORTRAN
*     CALL rayleigh( crs, error, lambda, n_lambda )
**PERL
*INPUT
**lambda
*     real array(*)
*     Wavelengths (nm) at which the Rayleigh scattering cross section 
*     is to be calculated.
**n_lambda
*     integer
*     Number of wavelengths lambda.
*OUTPUT
**error
*     integer
*     Error flag. 
**crs
*     real array(lambda)
*     The Rayleigh cross section as a function of lambda.
*ERROR_MESSAGES
*ROUTINES_CALLED
*FILES
*     consts.inc               Include file
*REFERENCES
**     M. Nicolet, 'On the molecular scattering in the terrestrial 
*         atmosphere: An empirical formula for its calculation in 
*         the homosphere', Planet. Space Sci., vol. 32, pp. 1467-1468,
*         1984.
*EOD
*
      INTEGER error, n_lambda
      REAL crs(*), lambda(*)
*
      REAL x, lambda_um, ray_const
      INCLUDE 'consts.inc'
      DATA ray_const /4.02E-28/
      SAVE ray_const
*
      error = 0
      DO iv = 1, n_lambda
         lambda_um = lambda(iv) * from_nm_um        
         IF ( lambda_um .GT. 0.55 )THEN
            x = 0.04
         ELSE
            x = 0.389*lambda_um + 0.09426/lambda_um - 0.3228
         ENDIF
         crs(iv)   = ray_const / (lambda_um**(4+x))
      ENDDO
*
      RETURN
*
      END
