      SUBROUTINE trans(aer_dtau, aer_gg, aer_ssa, albedo, dens, 
     $     dfdt, flup, ice_dtau, ice_gg, ice_ssa, lambda_t,
     $     ndens, nlambda_t, nlyr, nzout, path, press, rfldir, 
     $     rfldn, sza, temper, uavg, uavgdn, uavgso, uavgup,
     $     wc_dtau, wc_gg, wc_ssa, zd, zout)
*
      INCLUDE '../src_f/param.inc'
*
*BOD
*DESCRIPTION
* Calculates photodissociation rates.
*USAGE
**FORTRAN
*      CALL phodis(dens, ndens, nlyr, press,
*     $     temper, zd )  
**PERL
*INPUT
**dens
*     real array(levels,species)
*     The density of ndens species at nlyr+1 levels. The species index
*     must corresponds to the following trace gases if available. Water
*     vapor, carbon dioxide and nitrogen dioxide are presently not in
*     use and may be set to zero.
***1   Air
***2   ozone (o3)
***3   oxygen (o2)
***4   water vapor (h2o)
***5   carbon dioxide (co2)
***6   nitrogen dioxide (no2)
**ndens
*     integer
*     Number of densities in the model atmosphere. See dens.
**nlyr
*     integer
*     Number of layers in the model atmosphere.
**press
*     real array(0:*)
*     The pressure in mb at levels. 
**temper
*     real array(*)
*     The temperatures (levels in the atmosphere) at which
*     the ozone cross section is to be calculated.
**zd
*     real array(0:*)
*     The altitude in km for levels. 
*OUTPUT
*ERROR_MESSAGES
*ROUTINES_CALLED
*     crs_o3
*     rayleigh
*FILES
*     crs_o3_Molina.dat        Data file
*     consts.inc               Include file
*     param.inc                Include file
*REFERENCES
*EOD
*
      CHARACTER*(*) path     ! No. of characters must equal 
                             ! length of pathname            
      INTEGER ndens, nlambda_t, nlyr, nzout
      REAL aer_dtau(maxcly,maxwvn), aer_gg(maxcly,maxwvn), 
     $     aer_ssa(maxcly,maxwvn), albedo(*), dens(0:maxcly,*), 
     $     ice_dtau(maxcly,maxwvn), ice_gg(maxcly,maxwvn), 
     $     ice_ssa(maxcly,maxwvn), lambda_t(maxwvn), 
     $     press(0:maxcly), temper(0:maxcly),
     $     sza(*), wc_dtau(maxcly,maxwvn), wc_gg(maxcly,maxwvn), 
     $     wc_ssa(maxcly,maxwvn), zd(0:maxcly), zout(*)
*
* Internal variables
*
      INTEGER  error, type
      LOGICAL first, first_crs
      REAL abs_crs(0:maxcly, maxpro, maxwvn), 
     $     pi, ray_crs(maxwvn)
*
* Radiative transfer equation (RTE) solver parameters.
*
      CHARACTER header*127
      INTEGER ibcnd, ierror_d(47), ierror_s(33), imthd,
     $     ncoef, nil, nphi, nstr, numu
      LOGICAL deltam, newgeo, lamber, onlyfl, planck, 
     $     prndis(7), quiet, spher, usrtau
      REAL accur, beta(0:maxcly), btemp, dtauc(maxcly), 
     $     fbeam, fisot, HL( 0:MAXCMU ), phi(maxphi), 
     $     pmom(0:maxcmu,maxcly), radius, ssalb(maxcly), temis, 
     $     ttemp, umu(maxumu), umu0, utau(maxulv), wvnmhi, 
     $     wvnmlo
      REAL dfdt(maxulv, maxwvn), flup(maxulv, maxwvn),
     $     rfldir(maxulv, maxwvn), rfldn(maxulv, maxwvn), 
     $     uavg(maxulv, maxwvn), uavgdn(maxulv, maxwvn),
     $     uavgso(maxulv, maxwvn), uavgup(maxulv, maxwvn),
     $     ALBMED( MAXUMU ), TRNMED( MAXUMU )
      REAL u0u( maxumu, maxulv), uu( maxumu, maxulv, maxphi ) 
*
      DATA first /.TRUE./, first_crs /.TRUE./
      SAVE first, first_crs, pi
*
      IF ( first ) THEN
         pi = 2.0 * ASIN(1.0)
      ENDIF
*
*     SET VARIOUS INPUT DATA
*
      intgr       = 0
*
* Set RTE (Radiative Transfer Equation) solver input, technical stuff 
*
      accur       = 1.0E-3
      btemp       = 0.0
      deltam      = .TRUE.
      fbeam       = 1.0  ! Calculate only transmittance first.
      fisot       = 0.0
      header      = ' '
      ibcnd       = 0
      lamber      = .TRUE.
      newgeo      = .TRUE.
      nil         = 0
      nphi        = 0
      nstr        = 4
      ncoef       = nstr
      imthd       = 0
      onlyfl      = .TRUE.
      phi0        = 0.0
      planck      = .false.
      prndis(1)   = .false.
      prndis(2)   = .false.
      prndis(3)   = .false.
      prndis(4)   = .false.
      prndis(5)   = .false.
      prndis(6)   = .false.
      prndis(7)   = .false.
      quiet       = .false.
      radius      = 6370.0
      spher       = .TRUE.
      temis       = 0.0
      ttemp       = 0.0
      wvnmhi      = 0.0
      wvnmlo      = 0.0
cakyp
      write(*,*)"trans:", nlyr, nlambda_t, path
      stop
*      DO iv = 1, nlambda_t
*         WRITE(*,*)"TRANS", iv, lambda_t(iv)
*      ENDDO      
cakyp
*
*
* Ozone absorption
*
      type        = 1
      CALL crs_o3( abs_crs, error, first_crs, lambda_t, 
     $     nlambda_t, nlyr, path, temper, type)  
cakyp
*      lu = 1
*      DO iv = 1, nlambda_t
*         WRITE(12,*) lambda_t(iv), abs_crs(lu, io3, iv)
*      ENDDO
cakyp
*
* Rayleigh scattering
*
      CALL rayleigh( ray_crs, error, lambda_t, nlambda_t )
cakyp
      lu = 1
      DO iv = 1, nlambda_t
         WRITE(*,*)"TRANS", lambda_t(iv), abs_crs(lu, io3, iv),
     $        ray_crs(iv)
      ENDDO
      STOP
cakyp
*
* Calculate transmittance for all wavelengths nlambda_t
*
      IF ( nzout .EQ. 0 ) THEN
         nzout       = nlyr+1
         DO lu = 1, nzout
            zout(lu) = zd(lu-1)
         ENDDO
      ENDIF
      ntau        = nzout
      usrtau      = .TRUE.
*
      DO iv = 1, nlambda_t
cakyp
         write(12,*)"UMU0",iv, lambda_t(iv), sza(iv)
cakyp
         CALL opt_prop(abs_crs(0,1,iv), aer_dtau(1,iv), aer_gg(1,iv),
     $        aer_ssa(1,iv), dens, dtauc, ice_dtau(1,iv), ice_gg(1,iv),
     $        ice_ssa(1,iv), intgr, nlyr, nstr, pmom, ray_crs(iv),
     $        ssalb, wc_dtau(1,iv), wc_gg(1,iv), wc_ssa(1,iv), zd)
*
         umu0        = cos(sza(iv)*pi/180.)
cakyp
*            write(*,*)"UMU0",iv, lambda_t(iv), sza(iv), umu0
*            prndis(1) = .TRUE.
*            prndis(2) = .TRUE.
*            prndis(7) = .TRUE.
cakyp
         CALL setout( dtauc, nlyr, ntau, utau, zd, zout)
*
         CALL  disort( nlyr, dtauc, ssalb, pmom, ncoef, temper, 
     $     wvnmlo, wvnmhi, usrtau, ntau, utau, nstr, 
     $     usrang, numu, umu, nphi, phi, ibcnd, fbeam, 
     $     umu0, phi0, fisot, lamber, albedo(iv), hl, btemp, 
     $     ttemp, temis, deltam, imthd, planck, onlyfl, 
     $     accur, quiet, ierror_d, prndis, header, maxcly, 
     $     maxulv, maxumu, maxcmu, maxcoe, maxphi, 
     $     rfldir(1,iv), rfldn(1,iv), flup(1,iv), dfdt(1,iv), 
     $     uavg(1,iv), uu, u0u, albmed, trnmed, 
     $     uavgdn(1,iv), uavgso(1,iv), uavgup(1,iv) )
*
c$$$         CALL  sdisort( nlyr, dtauc, ssalb, pmom, temper, wvnmlo,
c$$$     $        wvnmhi, usrtau, ntau, utau, nstr, usrang, numu, 
c$$$     $        umu, nphi, phi, fbeam, beta, nil, umu0, phi0, 
c$$$     $        newgeo, zd, spher, radius, fisot, albedo(iv), btemp, 
c$$$     $        ttemp, temis, deltam, planck, onlyfl, accur, quiet, 
c$$$     $        ierror_s, prndis, header, maxcly, maxulv, maxumu, 
c$$$     $        maxcmu, maxphi, rfldir(1,iv), rfldn(1,iv), flup(1,iv),
c$$$     $        dfdt(1,iv), uavg(1,iv), 
c$$$     $        uu, u0u, uavgdn(1,iv), uavgso(1,iv), uavgup(1,iv) )
*     
      ENDDO
*
      RETURN
      END
